function [ boolean ] = create_html_summaries( p_nemo_data, p_config )
%CREATE_HTML_SUMMARY Generates a summary of every float given in the nemo data.
%   Uses the nemo configuration for linking the html summaries to the
%   overview table generated by create_csv_table
%
%   Parameters:
%   p_nemo_data:            The nemo data where the summaries will be
%                           generated from.
%   p_config:               The nemo configuration
%
%   Returns:
%   boolean:                Always true.

path = fullfile(p_config.ANALYSIS_RESULTS.output_path, 'html');

if (~exist(path, 'dir'))
    mkdir(path);
end

for i = 1:length(p_nemo_data)

    f = p_nemo_data{i};
    float_number = f{1}.FLOAT_IDENTIFICATION.float_serial_number;
    html = ['<!DOCTYPE html><head><title>Summary of Float ' ...
             num2str(float_number) ...
            '</title><style>' ...
            'body, a{font-family: Arial;color: #4B4B4D;}' ...
            'th{line-height: 60px;background: #00ACE5;color: #003E6E;font-size: 30px;}' ...
            'td{line-height: 40px;border: thin solid black; padding-left: 5px;}' ...
            '.value{width: 70%;}' ...
            '.name{width: 30%;background: #BCBDBF;font-weight: bold;}' ...
            'table{border-collapse: collapse;border: thin solid black;width: 100%;}' ...
            'img{width: 100%;font-weight: bold;font-size: 30px;}' ...
            '.overlay{z-index: 2;}' ...
            '#show-floattable{padding:20px;background: #00ACE5;color: #003E6E;font-size: 18px;position: fixed;right: 20px;top: 10px;cursor: pointer;}' ...
            '</style>' ...
            '</head><body>' ...
            '<a id="show-floattable" class="overlay" onclick="(function(){history.back();})()">Show all floats</a>' ...
            '<table>'];


    %% Add latest update time
    html = [ html '<tr><td class="name">Updated on:</td><td class="value">' datestr(datetime('now', 'TimeZone', 'Europe/London')) '</td></tr>'];


    %% Add float identification table
    html = [ html '<th colspan="2">FLOAT_IDENTIFICATION</th>' ];

    fnames = fieldnames(f{1}.FLOAT_IDENTIFICATION);
    for j = 1:length(fnames)

        fieldvalue = f{1}.FLOAT_IDENTIFICATION.(fnames{j});

        switch(class(fieldvalue))
            case {'double', 'logical'}
                fieldvalue = num2str(fieldvalue);
        end
        html = [ html ...
                 '<tr><td class="name">' fnames{j} ...
                 '</td><td class="value">' fieldvalue ...
                 '</td></tr>' ...
               ];
    end

    %% Add float deployment table
    html = [ html '<th colspan="2">DEPLOYMENT_INFO</th>' ];

    fnames = fieldnames(f{1}.DEPLOYMENT_INFO);
    for j = 1:length(fnames)

        fieldvalue = f{1}.DEPLOYMENT_INFO.(fnames{j});

        switch(class(fieldvalue))
            case {'double', 'logical'}
                fieldvalue = num2str(fieldvalue);
        end
        html = [ html ...
                 '<tr><td class="name">' fnames{j} ...
                 '</td><td class="value">' fieldvalue ...
                 '</td></tr>' ...
               ];
    end

    %% Add float deployment table
    html = [ html '<th colspan="2">OVERALL_MISSION_INFORMATION</th>' ];

    fnames = fieldnames(f{1}.OVERALL_MISSION_INFORMATION);
    for j = 1:length(fnames)

        fieldvalue = f{1}.OVERALL_MISSION_INFORMATION.(fnames{j});

        switch(class(fieldvalue))
            case {'double', 'logical'}
                fieldvalue = num2str(fieldvalue);
        end
        html = [ html ...
                 '<tr><td class="name">' fnames{j} ...
                 '</td><td class="value">' fieldvalue ...
                 '</td></tr>' ...
               ];
    end

    %% Add plots

    html = [ html '<th colspan="2">PLOTS</th>' ];

    html = [ html '<tr><td class="name">Arrival of profiles</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_ARRIVAL_OF_PROFILES.storage_directory_fig ...
             p_config.PLOT_ARRIVAL_OF_PROFILES.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_ARRIVAL_OF_PROFILES.storage_directory_image ...
             p_config.PLOT_ARRIVAL_OF_PROFILES.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
    html = [ html '<tr><td class="name">Ascent</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_ASCENT.storage_directory_fig ...
             p_config.PLOT_ASCENT.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_ASCENT.storage_directory_image ...
             p_config.PLOT_ASCENT.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
    html = [ html '<tr><td class="name">Categorized timings</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_CATEGORIZED_TIMINGS.storage_directory_fig ...
             p_config.PLOT_CATEGORIZED_TIMINGS.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_CATEGORIZED_TIMINGS.storage_directory_image ...
             p_config.PLOT_CATEGORIZED_TIMINGS.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
     html = [ html '<tr><td class="name">Timing information</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_TIMING_INFORMATION.storage_directory_fig ...
             p_config.PLOT_TIMING_INFORMATION.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_TIMING_INFORMATION.storage_directory_image ...
             p_config.PLOT_TIMING_INFORMATION.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
    html = [ html '<tr><td class="name">RAFOS</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_RAFOS.storage_directory_fig ...
             p_config.PLOT_RAFOS.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_RAFOS.storage_directory_image ...
             p_config.PLOT_RAFOS.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
    html = [ html '<tr><td class="name">Single floats map</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_SINGLE_FLOAT_SOUTH.storage_directory_fig ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_SINGLE_FLOAT_SOUTH.storage_directory_image ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
    html = [ html '<tr><td class="name">Temperature, Salinity</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_TEMP_SAL.storage_directory_fig ...
             p_config.PLOT_TEMP_SAL.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_TEMP_SAL.storage_directory_image ...
             p_config.PLOT_TEMP_SAL.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];
    html = [ html '<tr><td class="name">Temperature, Salinity, Potential Temperature</td>' ...
             '<td class="value"><a href="../../plots/' ...
             p_config.PLOT_TEMP_SAL_PTEMP.storage_directory_fig ...
             p_config.PLOT_TEMP_SAL_PTEMP.figure_filename ...
             num2str(float_number) '.fig">' ...
             '<img alt="No image available!" src="../../plots/' ...
             p_config.PLOT_TEMP_SAL_PTEMP.storage_directory_image ...
             p_config.PLOT_TEMP_SAL_PTEMP.image_filename ...
             num2str(float_number) '.png"></a>' ...
             '</td></tr>'];

    html = [ html '</table></body></html>' ];

    %% Write html file

    fid = fopen(fullfile(path, [num2str(float_number) '.html']), 'w');
    fwrite(fid, html);
    fclose(fid);

end

boolean = true;
end
