function [ boolean ] = save( p_profile, p_output_file, p_overwrite )
%SAVE Writes the input profile to the output file. 
%   This function saves the p_profile structure to an ASCII textfile with
%   .profile extension. The structure of the parameter is being transferred
%   equally into the textfile so that it can be loaded by the load
%   function.
%   TO BE OPTIMIZED: This function does not recursively checks for the
%   variable type. Only up to the second level.
%
%   Returns: Boolean variable, being false if something went wrong during
%            write.
%
%   Parameters:
%   p_profile:      Contains the profile structure that will be written to
%                   the output file.
%   p_output_file:  Absolute file path to the output file that will be
%                   written.
%   p_overwrite:    If set to true, the output file will be overwritten if
%                   it exists.
%   

%% Initialize return variable
boolean = false;

%% Parameter check
parameter_error = false;

if (isa(p_profile, 'logical') && ~p_profile)
    boolean = true;
    return
end

if (~isstruct(p_profile) || ~ischar(p_output_file))
    parameter_error = true;
end

if (isempty(p_overwrite))
    p_overwrite = 1; % default setting is to overwrite existing profile files
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% File existence checks

[directory, ~, ~] = fileparts(p_output_file);
if exist(directory, 'dir') ~= 7
    mkdir(directory)
end

if exist(p_output_file, 'file') == 2 && ~p_overwrite
    % file will not be overwritten
    boolean = true;
    return
end

%% Write file

% prepare for writing
help_tabs = ['\t', '\t', '\t', '\t', '\t', '\t', '\t', '\t', '\t', '\t', '\t', '\t'];
desired_tab_count = 12;
section_names = fieldnames(p_profile); % will be stored in between []
section_count = length(section_names);
output_file_id = fopen(p_output_file, 'w'); % open file for writing
double_format = '%.15g';

% write content
for o_section = 1:section_count % run through every section
    
    % write section name to file
    fprintf(output_file_id, ['[' section_names{o_section} ']' '\n' ]);
        
    % check which type of data it is and decide how to output
    switch class(p_profile.(section_names{o_section}))
        case 'struct'
            write_struct(p_profile.(section_names{o_section}));
            
        case 'cell'
            write_cell(p_profile.(section_names{o_section}));
            
        case 'double'
            write_double(p_profile.(section_names{o_section}));
            
        otherwise
    end
    
    fprintf(output_file_id, '\n');
    

end

% close file
fclose(output_file_id);

boolean = true; % update return variable


    function [ tabs ] = get_tabs_to_insert(p_member_name)
        % this function computes the required tabs to insert, making the
        % textfile easier to read.
        tablength = 4;
        tab_count = desired_tab_count - floor((length(p_member_name)+1)/tablength);
        tabs = help_tabs(1:(tab_count*2));
    end

    function write_struct(p_input_struct)
        % get all fieldnames and write them to the file
        member_names = fieldnames(p_input_struct);
        member_count = length(member_names);
        for o_member = 1:member_count % run through all members
            % calculate how many tabs have to be inserted to have a
            % good table layout
            tabstring = get_tabs_to_insert(member_names{o_member});
            fprintf(output_file_id, ['-' member_names{o_member} tabstring]);
            current_member = p_input_struct.(member_names{o_member});
            current_member_class = class(current_member);
            switch current_member_class
                case {'double', 'logical'}
                    if (isempty(current_member))
                        fprintf(output_file_id, [current_member '\n']);
                    else
                        write_double(current_member);
                    end
                case 'char'
                    fprintf(output_file_id, [current_member '\n']);
                otherwise
                    disp(['Unknown class in file: ' class(current_member)]);
            end

        end
        clear o_member
    end

    function write_cell(p_input_cell)
        % run through every cell member and check the class
        for o_cell_line = 1:size(p_input_cell, 1)
            for o_cell_column = 1:size(p_input_cell, 2)
                current_value = p_input_cell{o_cell_line, o_cell_column};
                switch class(current_value)
                    case 'char'
                        fprintf(output_file_id, current_value);
                    case {'double', 'logical'}
                        fprintf(output_file_id, double_format, current_value);
                    otherwise
                        disp(['Unknown class in file: ' class(current_value)]);
                end
                if (o_cell_column ~= size(p_input_cell, 2))
                    fprintf(output_file_id, '\t');
                end
            end
            
            fprintf(output_file_id, '\n');
        end
    end

    function write_double(p_input_double)
        
        for o_double = 1:size(p_input_double, 1)
            if length(p_input_double(o_double, :)) > 1
                fprintf(output_file_id, sprintf([double_format '\t'], p_input_double(o_double, 1:end-1)));
            end
            fprintf(output_file_id, [double_format '\n'], p_input_double(o_double, end));
        end
        clear o_double
        
    end


end

