function [ filtered_nemo_data ] = get_profiles_in_timerange( p_nemo_data, ...
    p_tlower, p_tupper )
%GET_PROFILES_IN_TIMERANGE Returns all profiles in the given time range.
%   This function returns all profiles that have been found in the given
%   time range. The check is being performed with the built in MATLAB function
%   isbetween. Please check documentation to be sure p_tlower and p_tupper is being
%   set correctly. The structure of p_nemo_data is being held, all profiles
%   that are not in the timrange are being removed. If p_nemo_data is a
%   string, it reads the profiles using nemo.load_data().
%
%   Returns: p_nemo_data with filtered profiles, false on fail.
%
%   Requirements:
%   - read_profiles.m: Containing function that reads nemo profiles from a
%       file into a multidimensional struct.
%
%   Parameters:
%   p_nemo_data:    Either has to be a multidimensional struct (created by
%                   nemo.load.data()) or a string, where the nemo float data
%                   is stored.
%   p_tlower, p_tupper:     Lower and upper time range boundary. To get examples how
%                   they should be set up, see "help inpolygon"


%% Initialize return variables

filtered_nemo_data = {};

%% Check parameters
parameter_error = false;

if (ischar(p_nemo_data)) % read profiles using read_profiles.m
    if (~isdir(p_nemo_data)) % parameter is not a directory
        parameter_error = true;
    else % read profiles
        p_nemo_data = nemo.load.data(p_nemo_data);
    end
else if (~iscell(p_nemo_data)) % raise error
        parameter_error = true;
    end
end


if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please provide either a ' ...
        'directory path or a struct created by read_profiles.m!']);
    return;
else % do some cleanup
    clear parameter_error;
end


%% Delete all profiles that are not in polygon

%nemo_field_names = fieldnames(p_nemo_data); % get all float names

for index_current_float = 1:length(p_nemo_data) % process every single float
    current_profiles = p_nemo_data{index_current_float};
    profile_count = length(current_profiles);
    filtered_profiles = {};
    
    % loop through current_profiles
    index_current_profile = 1;
    while (index_current_profile <= profile_count)
        % get position
        profile_time = datetime(current_profiles{index_current_profile}.SECOND_ORDER_INFORMATION.best_guess_datevec);
        
        if (isbetween(profile_time, p_tlower, p_tupper))
            % profile is not in given rectangle, so delete it
            filtered_profiles{end + 1} = current_profiles{index_current_profile};
        end
        
        % increase profile index
        index_current_profile = index_current_profile + 1;
    end
    
    % update return variable
    if (~isempty(filtered_profiles)) % there are profiles to update
        filtered_nemo_data{end + 1} = filtered_profiles;
    end
    
    clear current_profiles profile_field_names profile_count ...
        index_current_profile;
    
end

end
